package com.qs.mp.quartz.task;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.qs.mp.admin.domain.Marketing;
import com.qs.mp.admin.service.IMarketingAwardsService;
import com.qs.mp.admin.service.IMarketingService;
import com.qs.mp.common.core.redis.DistributedLocker;
import com.qs.mp.common.core.redis.RedisCache;
import com.qs.mp.common.enums.MarketingStatusEnum;
import com.qs.mp.common.utils.DateUtils;
import com.qs.mp.user.service.IMarketingUserCodeService;
import org.redisson.Redisson;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.Date;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * 营销活动相关任务
 *
 * @author Cup
 * @date 2022/5/18
 */
@Component("marketingTask")
public class MarketingTask {

    public static final String MARKETING_LOTTERY_KEY = "MARKETING_LOTTERY_KEY";


    @Autowired
    private DistributedLocker distributedLocker;

    @Autowired
    private IMarketingService marketingService;


    /**
     * 开奖任务方法
     */
    public void lottery() {

        // 加锁
        if (!distributedLocker.tryLock(MARKETING_LOTTERY_KEY)) {
            return;
        }
        try {
            // 获取结束时间小于等于当前时间且未开奖的一个活动
            Date now = DateUtils.getNowDate();
            Marketing marketing = marketingService.getOne(new LambdaQueryWrapper<Marketing>()
                    .eq(Marketing::getTriggerStatus, 0)
                    .eq(Marketing::getIsOn, MarketingStatusEnum.ON.getValue())
                    .le(Marketing::getEndTime, now)
                    .last("limit 1"));
            if (Objects.isNull(marketing)) {
                return;
            }

            // 开奖
            marketingService.lottery(marketing);


        }finally {
            // 释放锁
            distributedLocker.unlock(MARKETING_LOTTERY_KEY);
        }



    }


}
